from edge import eventlog
from edge.exceptions import NotFoundError


class UiRef:
    def __init__(self, ref):
        """
        :type ref: str
        """
        assert ref
        self._ref = ref

    def __eq__(self, other):
        return isinstance(other, self.__class__) and getattr(other, 'ref', None) == self.ref

    def __hash__(self):
        return hash(self.ref)

    def __str__(self):
        return self.ref

    @property
    def ref(self):
        """
        :rtype: str
        """
        return self._ref

    @property
    def type(self):
        """
        :rtype: str
        """
        segments = self.ref.split('~')
        subtypes = map(lambda x: x.split('.')[0], segments)
        return '~'.join(subtypes)

    def assert_type(self, other_type):
        """
        :type other_type: str
        :rtype: None

        :raises NotFoundError:
        """
        if self.type != other_type:
            eventlog.log_w('fail: NotFoundError ({})'.format(self.ref))
            raise NotFoundError

    @property
    def root_ref(self):
        """
        :rtype: str
        :raises NotFoundError:
        """
        return self.ref.split('~', 1)[0]

    @property
    def parent_ref(self):
        """
        :rtype: str
        :raises NotFoundError:
        """
        parent_ref = '~'.join(self.ref.split('~')[:-1])
        if not parent_ref:
            raise NotFoundError

        return parent_ref

    @property
    def parent(self):
        """
        :rtype: UiRef
        :raises NotFoundError:
        """
        return UiRef(self.parent_ref)

    def child_ref(self, child_type, child_chan):
        """
        :type child_type: str
        :type child_chan: str | int
        :rtype: str
        """
        if not child_type.startswith(self.type):
            raise NotFoundError

        child_type_segments = child_type.split('~')
        if not len(self.type.split('~')) == len(child_type_segments) - 1:
            raise NotFoundError

        return '{}~{}.{}'.format(self.ref, child_type_segments[-1], child_chan)

    def child(self, child_type, child_chan):
        """
        :type child_type: str
        :type child_chan: str | int
        :rtype: UiRef
        """
        return UiRef(self.child_ref(child_type, child_chan))
